import mythic_container
from mythic_container.logging import logger

MYTHIC_RPC_OTHER_SERVICES_RPC = "mythic_rpc_other_service_rpc"


class MythicRPCOtherServiceRPCMessage:
    def __init__(self,
                 ServiceName: str = None,
                 service_name: str = None,
                 ServiceRPCFunction: str = None,
                 service_function: str = None,
                 ServiceRPCFunctionArguments: dict = None,
                 service_arguments: dict = None,
                 **kwargs):
        self.ServiceName = ServiceName
        if self.ServiceName is None:
            self.ServiceName = service_name
        self.ServiceRPCFunction = ServiceRPCFunction
        if self.ServiceRPCFunction is None:
            self.ServiceRPCFunction = service_function
        self.ServiceRPCFunctionArguments = ServiceRPCFunctionArguments
        if self.ServiceRPCFunctionArguments is None:
            self.ServiceRPCFunctionArguments = service_arguments
        for k, v in kwargs.items():
            logger.error(f"unknown kwarg {k} {v}")
    def to_json(self):
        return {
            "service_name": self.ServiceName,
            "service_function": self.ServiceRPCFunction,
            "service_arguments": self.ServiceRPCFunctionArguments
        }
class MythicRPCOtherServiceRPCMessageResponse:
    def __init__(self,
                 success: bool = None,
                 error: str = None,
                 result: dict = None,
                 Success: bool = None,
                 Error: str = None,
                 Result: dict = None,
                 **kwargs):
        self.Success = Success
        if self.Success is None:
            self.Success = success
        self.Error = Error
        if self.Error is None:
            self.Error = error
        self.Result = Result
        if self.Result is None:
            self.Result = result
        for k, v in kwargs.items():
            logger.error(f"unknown kwarg {k} {v}")
    def to_json(self):
        return {
            "success": self.Success,
            "error": self.Error,
            "result": self.Result
        }


async def SendMythicRPCOtherServiceRPC(
        msg: MythicRPCOtherServiceRPCMessage) -> MythicRPCOtherServiceRPCMessageResponse:
    queue = f"{msg.ServiceName}_{MYTHIC_RPC_OTHER_SERVICES_RPC}"
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(queue=queue,
                                                                            body=msg.to_json())
    return MythicRPCOtherServiceRPCMessageResponse(**response)



import mythic_container
from mythic_container.logging import logger
from mythic_container.utils_mythic_file_transfer import getFileFromMythic

MYTHIC_RPC_PAYLOAD_GET_PAYLOAD_CONTENT = "mythic_rpc_payload_get_content"


class MythicRPCPayloadGetContentMessage:
    def __init__(self,
                 PayloadUUID: str,
                 **kwargs):
        self.PayloadUUID = PayloadUUID
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "uuid": self.PayloadUUID,
        }


class MythicRPCPayloadGetContentMessageResponse:
    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 content: bytes = None,
                 **kwargs):
        self.Success = success
        self.Error = error
        self.Content = content
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")


async def SendMythicRPCPayloadGetContent(
        msg: MythicRPCPayloadGetContentMessage) -> MythicRPCPayloadGetContentMessageResponse:
    content = await getFileFromMythic(agentFileId=msg.PayloadUUID)

    return MythicRPCPayloadGetContentMessageResponse(
        success=content is not None,
        error="Failed to fetch file from Mythic" if content is None else "",
        content=content
    )



from .send_mythic_rpc_agentstorage_create import *
from .send_mythic_rpc_agentstorage_remove import *
from .send_mythic_rpc_agentstorage_search import *
from .send_mythic_rpc_artifact_create import *
from .send_mythic_rpc_artifact_search import *
from .send_mythic_rpc_callback_add_command import *
from .send_mythic_rpc_callback_create import *
from .send_mythic_rpc_callback_decrypt_bytes import *
from .send_mythic_rpc_callback_encrypt_bytes import *
from .send_mythic_rpc_callback_remove_command import *
from .send_mythic_rpc_callback_search import *
from .send_mythic_rpc_callback_display_to_real_id_search import *
from .send_mythic_rpc_callback_search_command import *
from .send_mythic_rpc_callback_update import *
from .send_mythic_rpc_callbacktoken_create import *
from .send_mythic_rpc_callbacktoken_remove import *
from .send_mythic_rpc_command_search_ import *
from .send_mythic_rpc_credential_create import *
from .send_mythic_rpc_credential_search import *
from .send_mythic_rpc_file_create import *
from .send_mythic_rpc_file_get_content import *
from .send_mythic_rpc_file_register import *
from .send_mythic_rpc_file_search import *
from .send_mythic_rpc_file_update import *
from .send_mythic_rpc_filebrowser_create import *
from .send_mythic_rpc_filebrowser_remove import *
from .send_mythic_rpc_keylog_create import *
from .send_mythic_rpc_keylog_search import *
from .send_mythic_rpc_operationeventlog_create import *
from .send_mythic_rpc_payload_add_command import *
from .send_mythic_rpc_payload_create_from_scratch import *
from .send_mythic_rpc_payload_create_from_uuid import *
from .send_mythic_rpc_payload_get_content import *
from .send_mythic_rpc_payload_remove_command import *
from .send_mythic_rpc_payload_search import *
from .send_mythic_rpc_payload_update_build_step import *
from .send_mythic_rpc_payloadonhost_create import *
from .send_mythic_rpc_process_create import *
from .send_mythic_rpc_process_search import *
from .send_mythic_rpc_proxy_start import *
from .send_mythic_rpc_proxy_stop import *
from .send_mythic_rpc_response_create import *
from .send_mythic_rpc_response_search import *
from .send_mythic_rpc_task_create_subtask import *
from .send_mythic_rpc_task_create_subtask_group import *
from .send_mythic_rpc_task_display_to_real_id_search import *
from .send_mythic_rpc_task_search import *
from .send_mythic_rpc_task_update import *
from .send_mythic_rpc_token_create import *
from .send_mythic_rpc_token_remove import *



import mythic_container
from mythic_container.logging import logger

MYTHIC_RPC_PROXY_START = "mythic_rpc_proxy_start"


class MythicRPCProxyStartMessage:
    def __init__(self,
                 TaskID: int,
                 Port: int,
                 PortType: str,
                 **kwargs):
        self.TaskID = TaskID
        self.Port = Port
        self.PortType = PortType
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "task_id": self.TaskID,
            "port": self.Port,
            "port_type": self.PortType
        }


class MythicRPCProxyStartMessageResponse:
    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 **kwargs):
        self.Success = success
        self.Error = error
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")


async def SendMythicRPCProxyStartCommand(
        msg: MythicRPCProxyStartMessage) -> MythicRPCProxyStartMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(queue=MYTHIC_RPC_PROXY_START,
                                                                            body=msg.to_json())
    return MythicRPCProxyStartMessageResponse(**response)
